(function ($) {
$.Redactor.opts.langs['de'] = {
	html: 'HTML',
	video: 'Video',
	image: 'Bild',
	table: 'Tabelle',
	link: 'Link',
	link_insert: 'Link einfügen ...',
	link_edit: 'Link bearbeiten',
	unlink: 'Link entfernen',
	formatting: 'Formatvorlagen',
	paragraph: 'Absatz',
	quote: 'Zitat',
	code: 'Code',
	header1: 'Überschrift 1',
	header2: 'Überschrift 2',
	header3: 'Überschrift 3',
    header4: 'Überschrift 4',
    header5: 'Überschrift 5',
	bold:  'Fett',
	italic: 'Kursiv',
	fontcolor: 'Schriftfarbe',
	backcolor: 'Texthervorhebungsfarbe',
	unorderedlist: 'Aufzählungszeichen',
	orderedlist: 'Nummerierung',
	outdent: 'Einzug verkleinern',
	indent: 'Einzug vergrößern',
	redo: 'Wiederholen',
	undo: 'Rückgängig',
	cut: 'Ausschneiden',
	cancel: 'Abbrechen',
	insert: 'Einfügen',
	save: 'Speichern',
	_delete: 'Löschen',
	insert_table: 'Tabelle einfügen',
	insert_row_above: 'Zeile oberhalb einfügen',
	insert_row_below: 'Zeile unterhalb einfügen',
	insert_column_left: 'Spalte links einfügen',
	insert_column_right: 'Spalte rechts einfügen',
	delete_column: 'Spalte löschen',
	delete_row: 'Zeile löschen',
	delete_table: 'Tabelle löschen',
	rows: 'Zeilen',
	columns: 'Spalten',
	add_head: 'Titel einfügen',
	delete_head: 'Titel entfernen',
	title: 'Title',
    subtitle: 'Untertitel',
	image_view: 'Bilder',
	image_position: 'Textumbruch',
	none: 'Keine',
	left: 'Links',
	right: 'Rechts',
	image_web_link: 'Bilder-Link',
	text: 'Text',
	mailto: 'Email',
	web: 'URL',
	video_html_code: 'Video-Einbettungscode',
	file: 'Datei',
	upload: 'Hochladen',
	download: 'Download',
	choose: 'Auswählen',
	or_choose: 'Oder, wählen Sie eine Datei aus',
	drop_file_here: 'Ziehen Sie eine Datei hier hin',
	align_left:	'Linksbündig',
	align_center: 'Mitte',
	align_right: 'Rechtsbündig',
	align_justify: 'Blocksatz',
	horizontalrule: 'Horizontale Linie',
	fullscreen: 'Vollbild',
	deleted: 'Durchgestrichen',
	anchor: 'Anker',
	link_new_tab: 'Link in neuem Tab öffnen',
	underline: 'Unterstrichen',
	alignment: 'Ausrichtung',
	filename: 'Name (optional)',
	edit: 'Bearbeiten',
	center: 'Center',
	upload_label: 'Datei hier ablegen oder ',
	redo: 'Wiederherstellen',
	undo: 'Rückgängig'
};
})( jQuery );